/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
  * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
*********************************/

package fanchart;




import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Stroke;


public class Config {
	public static int db_distanceBetweenWindowEdgeAndOuterRadius;
	public static int baseWedgeThickness;
	public static int perGenerationThickness;
	public static int expansionWedgeThickness;
	public static int db_totalNumberOfRadii;
	public static int numberOfGenerationsToShow;
	public static int rootNodeRadius;
	public static Color textColor;
	public static Color backgroundColor;
	public static Color circleBackgroundColor;
	public static boolean antialias;
	public static boolean db_animate;
	public static boolean clickToExpand;
	public static Color wedgeBorderColor;
	public static Color generationColors[];
	public static String fontFamilyName;
	public static Font regularFont;
	public static int regularFontSize;
	public static Stroke regularLine;
	public static Stroke thickLine;
	public static Stroke thinLine;
	public static int db_millisecondsBetweenWedgeAnimationFrames;
	public static final float thicknessRatio = (float)Math.sqrt(2);
	public static final int DEFAULT_WINDOW_WIDTH = 850;
	public static final int DEFAULT_WINDOW_HEIGHT = 900;

	public static void init() {
		antialias = false;
		db_animate = true;
		clickToExpand = false;
		db_distanceBetweenWindowEdgeAndOuterRadius = 100;
		rootNodeRadius = 40;
		baseWedgeThickness = 30;
		perGenerationThickness = 10;
		setNumGenerations(10);
		expansionWedgeThickness = 15;
		generationColors = new Color[5];
		generationColors[0] = new Color(207,248,210);
		generationColors[1] = new Color(230,192,254);
		generationColors[2] = new Color(192,192,254);
		generationColors[3] = new Color(192,223,254);
		generationColors[4] = new Color(184,220,198);
		db_millisecondsBetweenWedgeAnimationFrames = 25;
		textColor = Color.BLACK;
		backgroundColor = new Color(248,248,248);
		circleBackgroundColor = new Color(244,239,227);
		wedgeBorderColor = Color.BLACK;
		fontFamilyName = "Arial";
		regularFontSize = 16;
		regularFont = new Font(fontFamilyName, Font.BOLD, regularFontSize);
		regularLine = new BasicStroke(1);
		thickLine = new BasicStroke(2);
		thinLine = new BasicStroke(1);
	}
	
	public static void setNumGenerations(int n) {
		numberOfGenerationsToShow = n;
		db_totalNumberOfRadii = numberOfGenerationsToShow+2;
	}
	
	public static Color getColorForPerson(Person p) {
		int n = p.getGeneration();
		while (n < 0) {
			n += generationColors.length;
		}
		return generationColors[n % generationColors.length];
	}
	
	//This sizes the wedges according to my original algorithm
	public static int getWedgeThicknessForGeneration(int g) {
		return baseWedgeThickness+perGenerationThickness*(g-1);
	}

	//This sizes the wedges the same way as PAF Companion
	public static int getNextWedgeThickness(float width) {
		return (int)(width*thicknessRatio);
	}
	
}
