/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class EditPersonDialog extends JDialog implements ActionListener {
	
	private Person person;
	private JTextField nameTextField;
	private static final long serialVersionUID = 42L;
	private static final String CANCEL_BUTTON_LABEL = "Cancel";
	private static final String OK_BUTTON_LABEL = "OK";	
	public EditPersonDialog(Dialog parent, boolean existing, Person guy) {
		super(parent, true);
		init(existing, guy, parent);
	}
	
	public EditPersonDialog(Frame parent, boolean existing, Person guy) {
		super(parent, true);
		init(existing, guy, parent);
	}
	
	private void init(boolean existing, Person guy, Component parent) {
		if (existing) {
			setTitle("Edit Person");
		} else {
			setTitle("Edit Person");
		}
		this.person = guy;
		buildGUI(existing);
		setResizable(false);
		pack();
		
		//position this dialog in the middle of its parent window
		Rectangle parentBounds = parent.getBounds();
		Dimension mySize = getSize();
		setLocation(parentBounds.x + parentBounds.width/2 - mySize.width/2,
				parentBounds.y + parentBounds.height/2 - mySize.height/2);

		
		//treat a window-closing event as if the user
		//had clicked the "cancel" button.
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent event) {
				handleActionEvent(CANCEL_BUTTON_LABEL);
			}
		});

		setVisible(true);

	}
	
	private void buildGUI(boolean editExisting) {
		setLayout(new BoxLayout(getContentPane(), BoxLayout.PAGE_AXIS));
		JPanel namePanel = new JPanel();
		JLabel nameLabel = new JLabel("Name: ");
		nameTextField = new JTextField();
		if (!editExisting) {
			//nameTextField.setText(Main.messages.getString(Constants.BROTHER) + " " + familyName);
			//nameTextField.select(0, Main.messages.getString(Constants.BROTHER).length());
		} else {
			nameTextField.setText(person.getName());
		}
		namePanel.add(nameLabel);
		namePanel.add(nameTextField);
		
		
		JPanel okCancelPanel = new JPanel();
		okCancelPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
		JButton okButton = new JButton(OK_BUTTON_LABEL);
		getRootPane().setDefaultButton(okButton);
		JButton cancelButton = new JButton(CANCEL_BUTTON_LABEL);
		okCancelPanel.add(okButton);
		okCancelPanel.add(cancelButton);
		
		okButton.setActionCommand(OK_BUTTON_LABEL);
		cancelButton.setActionCommand(CANCEL_BUTTON_LABEL);
		okButton.addActionListener(this);
		cancelButton.addActionListener(this);
		
		add(namePanel);
		add(okCancelPanel);
	}

	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		handleActionEvent(command);
	}
	
	private void handleActionEvent(String command) {
		if (command.equals(CANCEL_BUTTON_LABEL)) {
			person.setChanged(false);
		}
		if (command.equals(OK_BUTTON_LABEL)) {
			person.setName(nameTextField.getText());
			person.setChanged(true);
		}
		setVisible(false);
	}
}
