/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;


import java.io.Serializable;

/**
 * This class maintains a cache of vertices for use in drawing
 * the circle.  This saves some computation when rotating the
 * wedges of the circle, or adding and removing wedges because
 * no trigonometry needs to be recomputed.  The cache is only
 * refreshed when the window is resized.
 * Acknowledgement: This class was inspired by discussions with
 * Dr. Yarden Livnat (SCI Institute, University of Utah).
 * @author draperg
 *
 */
public class RadialCache implements Serializable {
	public static final int RESOLUTION = 1000; //MUST be > 0
	private java.awt.geom.Point2D.Float[][] vertices;
	private static final long serialVersionUID = 42L;
	private static final float dt = (float)(2*Math.PI / RESOLUTION);
	private static final int hack = RESOLUTION/2;

	
	public RadialCache() {
	}
	
	public void setRadii(final float radii[], final java.awt.geom.Point2D offset) {
		float theta, sine, cosine;
		float x,y;
		theta = 0;
		vertices = new java.awt.geom.Point2D.Float[Config.db_totalNumberOfRadii][RESOLUTION];
		for (int i=0; i<RESOLUTION; ++i) {
			sine = (float)Math.sin(theta);
			cosine = (float)Math.cos(theta);
			for (int j=0; j<Config.db_totalNumberOfRadii; ++j) {
				x = (radii[j] * sine) + (float)offset.getX();
				y = (-1 * radii[j] * cosine) + (float)offset.getY();
				vertices[j][i] = new java.awt.geom.Point2D.Float(x, y);
			}
			theta += dt;
		}
	}
	
	public java.awt.geom.Point2D.Float getVertex(final int radiusIndex, int degree) {
		while (degree < 0 || degree >= RESOLUTION) {
			degree = (degree + RESOLUTION) % RESOLUTION;
		}
		return vertices[radiusIndex][degree];
	}

	
	public static java.awt.geom.Point2D.Float getSpecialVertex(final float trueRadius,
			final int index, final java.awt.geom.Point2D.Float offset) {
		float theta = dt*(hack-index);
		float sine = (float)Math.sin(theta);
		float cosine = (float)Math.cos(theta);
		return new java.awt.geom.Point2D.Float(Math.round(trueRadius*sine)
				+ offset.x, Math.round(trueRadius*cosine) + offset.y);
	}
	
}
