/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications, for any purpose,
 * commercial or otherwise.
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.EventListener;
import java.util.List;

public class Ward implements Serializable {
	private List<WardListener> listeners;
	private static final long serialVersionUID = 42L;
	private Person rootPerson;
	private List<Person> linearListOfPeople;
	
	public Ward() {
		listeners = new ArrayList<WardListener>();
		linearListOfPeople = new ArrayList<Person>();
		rootPerson = null;
	}
	
	public boolean addWardListener(WardListener wl) {
		return listeners.add(wl);
	}
	
	public List<WardListener> clearWardListeners() {
		List<WardListener> backup = new ArrayList<WardListener>(listeners);
		listeners.clear();
		return backup;
	}
	
	public void restoreWardListeners(List<WardListener> backup) {
		listeners.clear();
		listeners.addAll(backup);
	}
	
	
	
	public void assignmentsChanged() {
		fireWardChanged();
	}
	
	private void fireWardChanged() {
		for (WardListener el : listeners) {
			el.wardChanged();
		}
	}
	
	public interface WardListener extends EventListener {
		public void wardChanged();
	}

	public Person setRootPerson(Person p) {
		rootPerson = p;
		return rootPerson;
	}
	
	public Person getRootPerson() {
		return rootPerson;
	}
	
	public void configureGenerations() {
		rootPerson.setGeneration(0);
		constructLinearList();
	}
	
	private void constructLinearList() {
		rootPerson.addToList(linearListOfPeople);
		Collections.sort(linearListOfPeople);
	}
	
	public List<Person> getLinearList() {
		return linearListOfPeople;
	}
	
	public Person getArbitaryPerson(int index) {
		return linearListOfPeople.get(index);
	}

	void testing() {
		Person p1 = new Person("Hannah Lily Baker");
		Person p2 = new Person("Adelle Anderson");
		Person p3 = new Person("Kevin Michael Baker");
		Person p4 = new Person("Daniel Baker");
		Person p5 = new Person("Laura Baker");
		Person p6 = new Person("Greg Baker");
		Person p7 = new Person("Anna Baker");
		Person p8 = new Person("Peter Baker");
		Person p9 = new Person("Paul Baker");
		Person p10 = new Person("John Paul Baker");
		Person p11 = new Person("Juleen Smith");
		Person p12 = new Person("David Paul Baker");
		Person p13 = new Person("Jo Anne Madison");
		Person p14 = new Person("Emily Lisa Archer");
		Person p15 = new Person("Tom Red Smith");
		Person p18 = new Person("James Peter Archer");
		Person p16 = new Person("Rachel Steet");
		Person p17 = new Person("James Smith");
		Person p19 = new Person("Lucy Red");
		Person p20 = new Person("Lillian Farmer");
		Person p21 = new Person("James F. Madison");
		Person p22 = new Person("Terry Baker");
		Person p23 = new Person("Esther Beckham");
		Person p24 = new Person("James Smith");
		Person p25 = new Person("Sarah Dexter");
		Person p26 = new Person("Joshua Red");
		Person p27 = new Person("Harriet Kemp");
		Person p28 = new Person("Ruel Archer");
		Person p29 = new Person("Edith Blair");
		Person p30 = new Person("David Streets");
		Person p31 = new Person("Phebie Williams");
		Person p32 = new Person("Zemira Baker");
		Person p33 = new Person("Olga Poulson");
		Person p34 = new Person("Samuel Beckham");
		Person p35 = new Person("Esther Ransome");
		Person p36 = new Person("James Madison");
		Person p37 = new Person("Alice Pin");
		Person p38 = new Person("Archibald Farmer");
		Person p39 = new Person("Mary Hansen");		
		Person p40 = new Person("Joseph Smith Anderson");
		Person p41 = new Person("Darcee Black");
		Person p42 = new Person("Michael John Anderson");
		Person p43 = new Person("Angela Anderson");
		Person p44 = new Person("Fred Anderson");
		Person p45 = new Person("Laura Golden");
		Person p46 = new Person("Guillaume Anderson");
		Person p47 = new Person("Marge Barnes");
		Person p48 = new Person("Richard Golden");
		Person p49 = new Person("Christy Johnston");
		Person p50 = new Person("Tanya Kay Head");
		Person p51 = new Person("David Shawn Wheat");
		Person p52 = new Person("Jan Monson");
		Person p53 = new Person("Ryan Glacier");
		Person p54 = new Person("Elizabeth Moss");
		Person p55 = new Person("Meredith Pume");
		new Person("Tom Baker", p4, p50);
		new Person("Maria Baker", p4, p50);
		new Person("Alison Baker", p4, p50);
		new Person("Joseph David Wheat", p51, p5);
		new Person("Sarah Wheat", p51, p5);
		new Person("Michael Wallace Wheat", p51, p5);
		new Person("Edmund Wheat", p51, p5);
		new Person("Sirius Wheat", p51, p5);
		new Person("Benjamin Baker", p6, p52);
		new Person("Peter Baker", p6, p52);
		new Person("Bethany Baker", p6, p52);
		new Person("Nathan Baker", p6, p52);
		new Person("Rebecca Marie Glacier", p53, p7);
		new Person("Matthew Glacier", p53, p7);
		new Person("Hannah Glacier", p53, p7);
		new Person("David Brian Baker", p8, p54);
		new Person("Christopher Baker", p9, p55);
		new Person("Anthony Anderson", p40, p41);
		new Person("Aaron Michael Anderson", p40, p41);
		
		p1.setParents(p3, p2);
		p4.setParents(p10, p11);
		p5.setParents(p10, p11);
		p6.setParents(p10, p11);
		p3.setParents(p10, p11);
		p7.setParents(p10, p11);
		p8.setParents(p10, p11);
		p9.setParents(p10, p11);
		p10.setParents(p12, p13);
		p11.setParents(p15, p14);
		p15.setParents(p17, p19);
		p14.setParents(p18, p16);
		p40.setParents(p44, p45);
		p2.setParents(p44, p45);
		p42.setParents(p44, p45);
		p43.setParents(p44, p45);
		p44.setParents(p46, p47);
		p45.setParents(p48, p49);
		p12.setParents(p22, p23);
		p13.setParents(p21, p20);
		p17.setParents(p24, p25);
		p19.setParents(p26, p27);
		p16.setParents(p30, p31);
		p18.setParents(p28, p29);
		p22.setParents(p32, p33);
		p23.setParents(p34, p35);
		p21.setParents(p36, p37);
		p20.setParents(p38, p39);

		setRootPerson(p23);
	}
}
